package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.ScoreRuleShareRecord.ScoreRuleShareRecordDO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.ScoreRuleShareRecord.ScoreRuleShareRecordService;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.aop.framework.AopContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.GET_LOCK_FAILED;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class ShareStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private MemberUserService memberUserService;
    @Resource
    private ScoreRuleShareRecordService scoreRuleShareRecordService;
    @Resource
    private RedissonClient redissonClient;

    public ShareStrategy(ScoreRuleService scoreRuleService, MemberUserScoreLogService memberUserScoreLogService, MemberUserService memberUserService, MemberUserScoreApi memberUserScoreApi) {
        super(scoreRuleService, memberUserScoreLogService, memberUserService, memberUserScoreApi);
    }

    @Override
    public void addScore(ScoreMessage message) {
        Long userId = message.getUserId();
        Long shareRuleId = message.getShareRuleId();
        log.info("Received Share message,memberId:{},shareRuleId:{}", userId, shareRuleId);
        // 根据分享人和规则id获取分享记录
        ScoreRuleShareRecordDO scoreRuleShareRecordDO = scoreRuleShareRecordService.selectOne("member_id", userId, "rule_id", shareRuleId);
        if (scoreRuleShareRecordDO == null) {
            log.info("Share listener: The user not share the activity,userId:{},ruleId:{}", userId, shareRuleId);
            return;
        }
        // 获取分享规则信息
        ScoreRuleDO scoreRuleDO = scoreRuleService.getById(shareRuleId);
        if (scoreRuleDO == null) {
            log.info("Share rule is not exist,shareRuleId:{}", shareRuleId);
            return;
        }
        // 校验规则
        if (!verifyScoreRule(scoreRuleDO)) {
            return;
        }
        // 校验累计最高分
        Integer userScoreTotalCount = getUserScoreTotalCount(scoreRuleDO.getId(), userId);
        if (scoreRuleDO.getMaxScoreTotal() > 0 && userScoreTotalCount >= scoreRuleDO.getMaxScoreTotal()) {
            log.info("Share listener: The user has reached the maximum score,userId:{},scoreRuleId:{}", userId, scoreRuleDO.getId());
            return;
        }
        // 更新积分和分享记录
        String lockKey = "score:rule:share:lock:" + scoreRuleShareRecordDO.getId();
        RLock lock = redissonClient.getLock(lockKey);
        try {
            if (!lock.tryLock(2, 10, TimeUnit.SECONDS)) {
                throw exception(GET_LOCK_FAILED);
            }
            ShareStrategy currentProxy = (ShareStrategy) AopContext.currentProxy();
            currentProxy.executeScoreRule(message.getTriggerIp(), shareRuleId, userId, scoreRuleDO, scoreRuleShareRecordDO);
        } catch (InterruptedException e) {
            throw exception(GET_LOCK_FAILED);
        } finally {
            lock.unlock();
        }
        log.info("Share rule add score success,ruleID:{},userID:{}", scoreRuleDO.getId(), userId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void executeScoreRule(String triggerIp, Long shareRuleId, Long userId, ScoreRuleDO scoreRuleDO, ScoreRuleShareRecordDO scoreRuleShareRecordDO) {
        scoreRuleShareRecordDO.setTotalScore(scoreRuleShareRecordDO.getTotalScore() + scoreRuleDO.getGetScoreOnce());
        scoreRuleShareRecordDO.setLastTriggerTime(new Date());
        scoreRuleShareRecordService.updateById(scoreRuleShareRecordDO);
        HashMap<String, Object> map = new HashMap<>(1);
        map.put("scoreRuleId", shareRuleId);
        memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                .memberId(userId)
                .scoreCount(scoreRuleDO.getGetScoreOnce())
                .sourceType(ScoreSourceTypeEnum.SHARE)
                .ruleId(scoreRuleDO.getId())
                .uniqueId(userId + "_" + shareRuleId + "_" + triggerIp)
                .expireDays(scoreRuleDO.getScorePeriod())
                .extParam(map)
                .build());
    }

    private boolean verifyScoreRule(ScoreRuleDO scoreRuleDO) {
        if (scoreRuleDO.getStatus() != 1) {
            log.info("Share listener: The rule is not enable,ruleId:{}", scoreRuleDO.getId());
            return false;
        }
        if (scoreRuleDO.getType() != ScoreRuleTypeEnum.SHARE.getValue()) {
            log.info("Share listener: Not share rule,ruleId:{}", scoreRuleDO.getId());
            return false;
        }
        Date date = new Date();
        if (date.before(scoreRuleDO.getStartTime()) || date.after(scoreRuleDO.getEndTime())) {
            log.info("Share listener: The rule is not in the time range,ruleId:{}", scoreRuleDO.getId());
            return false;
        }
        return true;
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.SHARE;
    }
}
