package cn.iocoder.yudao.module.member.service.auth;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.monitor.TracerUtils;
import cn.iocoder.yudao.framework.common.util.servlet.ServletUtils;
import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.util.spring.enums.UserTypeEnum;
import cn.iocoder.yudao.framework.common.util.validation.PhoneUtil;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.i18n.core.LangEnum;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.authentication.MultiUsernamePasswordAuthenticationToken;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerMapper;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.CustomerOperateLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.api.internalMessage.ClientInternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.ecw.api.paramValid.ParamValidatorApi;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.member.api.ScoreProducerApi;
import cn.iocoder.yudao.module.member.controller.app.auth.vo.*;
import cn.iocoder.yudao.module.member.convert.auth.AuthConvert;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.dal.mysql.user.MemberUserMapper;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.UserOperationLogTypeEnum;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import cn.iocoder.yudao.module.member.service.userOperationLog.UserOperationLogService;
import cn.iocoder.yudao.module.member.vo.userOperationLog.UserOperationLogCreateReqVO;
import cn.iocoder.yudao.module.system.api.auth.UserSessionApi;
import cn.iocoder.yudao.module.system.api.logger.LoginLogApi;
import cn.iocoder.yudao.module.system.api.logger.dto.LoginLogCreateReqDTO;
import cn.iocoder.yudao.module.system.api.sms.SmsCodeApi;
import cn.iocoder.yudao.module.system.api.social.SocialUserApi;
import cn.iocoder.yudao.module.system.controller.admin.mail.vo.template.MailTemplateSendReqVO;
import cn.iocoder.yudao.module.system.enums.logger.LoginLogTypeEnum;
import cn.iocoder.yudao.module.system.enums.logger.LoginResultEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsIsOrdersEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsIsTransportEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsMessageTypeEnum;
import cn.iocoder.yudao.module.system.enums.sms.SmsNodeEnum;
import cn.iocoder.yudao.module.system.service.mail.MailSendService;
import com.google.common.annotations.VisibleForTesting;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.servlet.ServletUtils.getClientIP;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员的认证 Service 接口
 *
 * @author 捷道源码
 */
@Service
@Slf4j
public class MemberAuthServiceImpl implements MemberAuthService {

    @Resource
    @Lazy // 延迟加载，因为存在相互依赖的问题
    private AuthenticationManager authenticationManager;

    @Resource
    private MemberUserService userService;

    @Resource
    private MailSendService mailSendService;
    @Resource
    private SmsCodeApi smsCodeApi;
    @Resource
    private LoginLogApi loginLogApi;
    @Resource
    private UserSessionApi userSessionApi;
    @Resource
    private SocialUserApi socialUserApi;

    @Resource
    private PasswordEncoder passwordEncoder;
    @Resource
    private MemberUserMapper userMapper;

    @Resource
    private ClientInternalMessageApi clientInternalMessageApi;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private ParamValidatorApi paramValidatorApi;
    @Resource
    private ScoreProducerApi scoreProducerApi;

    @Resource
    private CustomerMapper customerMapper;

    @Resource
    private CustomerOperateLogService customerOperateLogService;

    @Override
    public UserDetails loadUserByUsername(String mobile) throws UsernameNotFoundException {
        // 获取 username 对应的 SysUserDO
        MemberUserDO user = userService.getUserByMobile(mobile);
        if (user == null) {
            throw new UsernameNotFoundException(mobile);
        }
        // 创建 LoginUser 对象
        return AuthConvert.INSTANCE.convert(user);
    }

    @Override
    public String reg(@Valid AppAuthRegReqVO reqVO, String userIp, String userAgent, Boolean isAutoLogin) {
        // 校验验证码
        smsCodeApi.useSmsCode(AuthConvert.INSTANCE.convert(reqVO, SmsNodeEnum.MEMBER_REG.getNodeValue(), userIp));

        MemberUserDO userByMobile = userService.checkPhoneUnique(reqVO.getAreaCode(), reqVO.getMobile());
        if (userByMobile != null) {
            throw exception(MEMBER_USER_MOBILE_EXIST);
        }

        // 获得获得注册用户
        MemberUserDO user = userService.createUser(reqVO.getMobile(), userIp, reqVO.getPassword(), reqVO.getAreaCode(), reqVO.getEnglishName(), reqVO.getReferralCode(), reqVO.getRegisterPlatform());
        Assert.notNull(user, "创建用户失败，结果为空");

        //通过memberUserId获取Customer
        CustomerDO customer = customerMapper.getByMemberUserId(user.getId());
        if (customer!=null) {
            CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                    .setOperator(1l)
                    .setOperatorName("admin")
                    .setCustomerId(customer.getId())
                    .setNumber(customer.getNumber())
                    .setName(customer.getName())
                    .setOperateType(CustomerOperateTypeEnum.VERIFICATION_CODE.getValue())
                    .setRemark("手机注册成功");
            //记录日志
            customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
        }

        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(user.getId())
                .setType(UserOperationLogTypeEnum.REGIST.getValue())
                .setTitle(UserOperationLogTypeEnum.REGIST.getDesc())
                .setIp(getClientIP())
                .setContent("手机注册成功");
        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);

        if (!isAutoLogin) {
            return null;
        }
        // 执行登陆
        this.createLoginLog(user.getMobile(), LoginLogTypeEnum.LOGIN_USERNAME, LoginResultEnum.SUCCESS);
        LoginUser loginUser = AuthConvert.INSTANCE.convert(user);

        // 缓存登录用户到 Redis 中，返回 sessionId 编号
        return userSessionApi.createUserSession(loginUser, userIp, userAgent);
    }

    @Override
    public boolean checkPhoneUnique(String areaCode, String mobile) {
        MemberUserDO memberUserDO = userService.checkPhoneUnique(areaCode, mobile);
        return memberUserDO != null;
    }

    @Override
    public String login(AppAuthLoginReqVO reqVO, String userIp, String userAgent) {
        // 使用手机 + 密码，进行登录。
        LoginUser loginUser = this.login0(reqVO.getMobile(), reqVO.getPassword());

        // 缓存登录用户到 Redis 中，返回 sessionId 编号
        return userSessionApi.createUserSession(loginUser, userIp, userAgent);
    }

    @Override
    @Transactional
    public String smsLogin(AppAuthSmsLoginReqVO reqVO, String userIp, String userAgent) {
        // 校验验证码
        smsCodeApi.useSmsCode(AuthConvert.INSTANCE.convert(reqVO, SmsNodeEnum.MEMBER_LOGIN.getNodeValue(), userIp));

        // 获得获得注册用户
        MemberUserDO user = userService.createUserIfAbsent(reqVO.getMobile(), reqVO.getAreaCode(), userIp, reqVO.getLoginPlatform());
        Assert.notNull(user, "获取用户失败，结果为空");

        // 执行登陆
        this.createLoginLog(user.getMobile(), LoginLogTypeEnum.LOGIN_SMS, LoginResultEnum.SUCCESS);
        LoginUser loginUser = AuthConvert.INSTANCE.convert(user);

        // 缓存登录用户到 Redis 中，返回 sessionId 编号
        return userSessionApi.createUserSession(loginUser, userIp, userAgent);
    }

    @Override
    public String socialLogin(AppAuthSocialLoginReqVO reqVO, String userIp, String userAgent) {
        // 使用 code 授权码，进行登录。然后，获得到绑定的用户编号
        Long userId = socialUserApi.getBindUserId(UserTypeEnum.MEMBER.getValue(), reqVO.getType(),
                reqVO.getCode(), reqVO.getState());
        if (userId == null) {
            throw exception(AUTH_THIRD_LOGIN_NOT_BIND);
        }

        // 自动登录
        MemberUserDO user = userService.getUser(userId);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        this.createLoginLog(user.getMobile(), LoginLogTypeEnum.LOGIN_SOCIAL, LoginResultEnum.SUCCESS);

        // 创建 LoginUser 对象
        LoginUser loginUser = AuthConvert.INSTANCE.convert(user);

        // 绑定社交用户（更新）
        socialUserApi.bindSocialUser(AuthConvert.INSTANCE.convert(loginUser.getId(), getUserType().getValue(), reqVO));

        // 缓存登录用户到 Redis 中，返回 sessionId 编号
        return userSessionApi.createUserSession(loginUser, userIp, userAgent);
    }

    @Override
    public String socialLogin2(AppAuthSocialLogin2ReqVO reqVO, String userIp, String userAgent) {
        // 校验社交平台的认证信息是否正确
        socialUserApi.checkSocialUser(reqVO.getType(), reqVO.getCode(), reqVO.getState());

        // 使用手机号、手机验证码登录
        AppAuthSmsLoginReqVO loginReqVO = AppAuthSmsLoginReqVO.builder()
                .mobile(reqVO.getMobile()).code(reqVO.getSmsCode()).build();
        String sessionId = this.smsLogin(loginReqVO, userIp, userAgent);
        LoginUser loginUser = userSessionApi.getLoginUser(sessionId);

        // 绑定社交用户（新增）
        socialUserApi.bindSocialUser(AuthConvert.INSTANCE.convert(loginUser.getId(), getUserType().getValue(), reqVO));
        return sessionId;
    }

    @Override
    public void socialBind(Long userId, AppAuthSocialBindReqVO reqVO) {
        // 绑定社交用户（新增）
        socialUserApi.bindSocialUser(AuthConvert.INSTANCE.convert(userId, getUserType().getValue(), reqVO));
    }

    @Override
    public void unbindSocialUser(Long userId, AppAuthSocialUnbindReqVO reqVO) {
        socialUserApi.unbindSocialUser(AuthConvert.INSTANCE.convert(userId, getUserType().getValue(), reqVO));
    }

    @Override
    public String getSocialAuthorizeUrl(Integer type, String redirectUri) {
        return socialUserApi.getAuthorizeUrl(type, redirectUri);
    }

    private LoginUser login0(String username, String password) {
        final LoginLogTypeEnum logType = LoginLogTypeEnum.LOGIN_USERNAME;
        // 用户验证
        Authentication authentication;
        try {
            // 调用 Spring Security 的 AuthenticationManager#authenticate(...) 方法，使用账号密码进行认证
            // 在其内部，会调用到 loadUserByUsername 方法，获取 User 信息
            authentication = authenticationManager.authenticate(new MultiUsernamePasswordAuthenticationToken(
                    username, password, getUserType()));
        } catch (BadCredentialsException badCredentialsException) {
            this.createLoginLog(username, logType, LoginResultEnum.BAD_CREDENTIALS);
            throw exception(AUTH_LOGIN_BAD_CREDENTIALS);
        } catch (DisabledException disabledException) {
            this.createLoginLog(username, logType, LoginResultEnum.USER_DISABLED);
            throw exception(AUTH_LOGIN_USER_DISABLED);
        } catch (AuthenticationException authenticationException) {
            log.error("[login0][username({}) 发生未知异常]", username, authenticationException);
            this.createLoginLog(username, logType, LoginResultEnum.UNKNOWN_ERROR);
            throw exception(AUTH_LOGIN_FAIL_UNKNOWN);
        }
        // 登录成功的日志
        Assert.notNull(authentication.getPrincipal(), "Principal 不会为空");
        this.createLoginLog(username, logType, LoginResultEnum.SUCCESS);
        return (LoginUser) authentication.getPrincipal();
    }

    private void createLoginLog(String mobile, LoginLogTypeEnum logType, LoginResultEnum loginResult) {
        // 获得用户
        MemberUserDO user = userService.getUserByMobile(mobile);
        // 插入登录日志
        LoginLogCreateReqDTO reqDTO = new LoginLogCreateReqDTO();
        reqDTO.setLogType(logType.getType());
        reqDTO.setTraceId(TracerUtils.getTraceId());
        if (user != null) {
            reqDTO.setUserId(user.getId());
        }
        reqDTO.setUserType(getUserType().getValue());
        reqDTO.setUsername(mobile);
        reqDTO.setUserAgent(ServletUtils.getUserAgent());
        reqDTO.setUserIp(getClientIP());
        reqDTO.setResult(loginResult.getResult());
        loginLogApi.createLoginLog(reqDTO);
        // 更新最后登录时间
        if (user != null && Objects.equals(LoginResultEnum.SUCCESS.getResult(), loginResult.getResult())) {
            // 注册，推荐积分规则触发判断
            scoreRuleCheck(user);

            userService.updateUserLogin(user.getId(), getClientIP());
        }

        //记录日志
        if (user != null) {
            UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
            userOperationLogCreateReqVO
                    .setUserId(user.getId())
                    .setType(UserOperationLogTypeEnum.LOGIN.getValue())
                    .setTitle(UserOperationLogTypeEnum.LOGIN.getDesc())
                    .setIp(getClientIP())
                    .setContent("登录成功");

            userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);
        }
    }

    @Override
    public LoginUser verifyTokenAndRefresh(String token) {
        // 获得 LoginUser
        LoginUser loginUser = userSessionApi.getLoginUser(token);
        if (loginUser == null) {
            return null;
        }
        // 刷新 LoginUser 缓存
        this.refreshLoginUserCache(token, loginUser);
        return loginUser;
    }

    private void refreshLoginUserCache(String token, LoginUser loginUser) {
        // 每 1/3 的 Session 超时时间，刷新 LoginUser 缓存
        if (System.currentTimeMillis() - loginUser.getUpdateTime().getTime() <
                userSessionApi.getSessionTimeoutMillis() / 3) {
            return;
        }

        // 重新加载 UserDO 信息
        MemberUserDO user = userService.getUser(loginUser.getId());
        if (user == null || CommonStatusEnum.DISABLE.getStatus().equals(user.getStatus())) {
            // 校验 token 时，用户被禁用的情况下，也认为 token 过期，方便前端跳转到登录界面
            throw exception(AUTH_TOKEN_EXPIRED);
        }

        // 刷新 LoginUser 缓存
        userSessionApi.refreshUserSession(token, loginUser);
    }

    @Override
    public LoginUser mockLogin(Long userId) {
        // 获取用户编号对应的 UserDO
        MemberUserDO user = userService.getUser(userId);
        if (user == null) {
            throw new UsernameNotFoundException(String.valueOf(userId));
        }

        // 执行登陆
        this.createLoginLog(user.getMobile(), LoginLogTypeEnum.LOGIN_MOCK, LoginResultEnum.SUCCESS);

        // 创建 LoginUser 对象
        return AuthConvert.INSTANCE.convert(user);
    }

    @Override
    public void logout(String token) {
        // 查询用户信息
        LoginUser loginUser = userSessionApi.getLoginUser(token);
        if (loginUser == null) {
            return;
        }
        // 删除 session
        userSessionApi.deleteUserSession(token);
        // 记录登出日志
        this.createLogoutLog(loginUser.getId(), loginUser.getUsername());
    }

    @Override
    public UserTypeEnum getUserType() {
        return UserTypeEnum.MEMBER;
    }

    @Override
    public void updatePassword(Long userId, AppAuthUpdatePasswordReqVO reqVO) {
        // 检验旧密码
        MemberUserDO userDO = checkOldPassword(userId, reqVO.getOldPassword());

        // 更新用户密码
        userMapper.updateById(MemberUserDO.builder().id(userDO.getId())
                .password(passwordEncoder.encode(reqVO.getPassword())).build());

        //修改密码站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userId))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.update.password.title", LangEnum.ZH.getLang()))
                .titleEn(I18nMessage.getMessage("app.message.update.password.title", LangEnum.EN.getLang()))
                .titleFr(I18nMessage.getMessage("app.message.update.password.title", LangEnum.FR.getLang()))
                .content(I18nMessage.getMessage("app.message.update.password.content", LangEnum.ZH.getLang()))
                .contentEn(I18nMessage.getMessage("app.message.update.password.content", LangEnum.EN.getLang()))
                .contentFr(I18nMessage.getMessage("app.message.update.password.content", LangEnum.FR.getLang()))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);
    }

    @Override
    public void resetPassword(AppAuthResetPasswordReqVO reqVO) {
        // 检验用户是否存在
        MemberUserDO userDO = checkUserIfExists(reqVO.getMobile());

        // 使用验证码
        smsCodeApi.useSmsCode(AuthConvert.INSTANCE.convert(reqVO, SmsNodeEnum.MEMBER_FORGET_PASSWORD,
                getClientIP()));

        // 更新密码
        userMapper.updateById(MemberUserDO.builder().id(userDO.getId())
                .password(passwordEncoder.encode(reqVO.getPassword())).build());

        //修改密码站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userDO.getId()))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.update.password.title", LangEnum.ZH.getLang()))
                .titleEn(I18nMessage.getMessage("app.message.update.password.title", LangEnum.EN.getLang()))
                .titleFr(I18nMessage.getMessage("app.message.update.password.title", LangEnum.FR.getLang()))
                .content(I18nMessage.getMessage("app.message.update.password.content", LangEnum.ZH.getLang()))
                .contentEn(I18nMessage.getMessage("app.message.update.password.content", LangEnum.EN.getLang()))
                .contentFr(I18nMessage.getMessage("app.message.update.password.content", LangEnum.FR.getLang()))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);
    }

    @Override
    public void updateControlPassword(Long userId, AppAuthUpdatePasswordReqVO reqVO) {
        // 检验旧密码
        MemberUserDO userDO = checkOldControlPassword(userId, reqVO.getOldPassword());

        // 更新用户密码
        userMapper.updateById(MemberUserDO.builder().id(userDO.getId())
                .controlPassword(passwordEncoder.encode(reqVO.getPassword())).build());

        //修改密码站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userId))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.ZH.getLang()))
                .titleEn(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.EN.getLang()))
                .titleFr(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.FR.getLang()))
                .content(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.ZH.getLang()))
                .contentEn(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.EN.getLang()))
                .contentFr(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.FR.getLang()))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);
    }

    @Override
    public void resetControlPassword(AppAuthResetPasswordReqVO reqVO) {
        // 检验用户是否存在
        MemberUserDO userDO = checkUserIfExists(reqVO.getMobile());

        // 使用验证码
        smsCodeApi.useSmsCode(AuthConvert.INSTANCE.convert(reqVO, SmsNodeEnum.MEMBER_FORGET_PASSWORD,
                getClientIP()));

        // 更新密码
        userMapper.updateById(MemberUserDO.builder().id(userDO.getId())
                .controlPassword(passwordEncoder.encode(reqVO.getPassword())).build());

        //修改密码站内信
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(userDO.getId()))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.ZH.getLang()))
                .titleEn(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.EN.getLang()))
                .titleFr(I18nMessage.getMessage("app.message.update.control.password.title", LangEnum.FR.getLang()))
                .content(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.ZH.getLang()))
                .contentEn(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.EN.getLang()))
                .contentFr(I18nMessage.getMessage("app.message.update.control.password.content", LangEnum.FR.getLang()))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);
        //记录日志
        UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
        userOperationLogCreateReqVO
                .setUserId(userDO.getId())
                .setType(UserOperationLogTypeEnum.RELEASE.getValue())
                .setTitle(UserOperationLogTypeEnum.RELEASE.getDesc())
                .setIp(getClientIP())
                .setContent("修改放货密码成功");
        //通过memberUserId获取Customer
        CustomerDO customer = customerMapper.getByMemberUserId(userDO.getId());
        if (customer!=null) {
            CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                    .setOperator(1l)
                    .setOperatorName("admin")
                    .setCustomerId(customer.getId())
                    .setNumber(customer.getNumber())
                    .setName(customer.getName())
                    .setOperateType(CustomerOperateTypeEnum.VERIFICATION_CODE.getValue())
                    .setRemark("修改放货密码成功");
            customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
        }

        //记录日志
        userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);
    }

    @Override
    public void sendSmsCode(Long userId, AppAuthSendSmsReqVO reqVO) {
        if (StringUtils.isBlank(reqVO.getNodeValue())) {
            // 旧版本登录处理
            if (reqVO.getScene()!=null&&reqVO.getScene()==1) {
                reqVO.setIsOrders(SmsIsOrdersEnum.SMS_ORDERS_0.getValue());
                reqVO.setIsTransport(SmsIsTransportEnum.SMS_IS_TRANSPORT_0.getValue());
                reqVO.setMessageType(SmsMessageTypeEnum.SMS_MESSAGE_TYPE_1.getValue());
                reqVO.setNodeValue(SmsNodeEnum.MEMBER_LOGIN.getNodeValue());
                // 其他
                reqVO.setTransportId(0);
            }
        }
        String mobileCode = reqVO.getAreaCode() + StrUtil.COLON + reqVO.getMobile();
        paramValidatorApi.validatorMobile(mobileCode);
        String mobile = PhoneUtil.formatPhone(mobileCode);
        if (StringUtils.isNotBlank(mobile)) {
            reqVO.setMobile(mobile);
        }
        if (reqVO.getNodeValue().equals(SmsNodeEnum.MEMBER_FORGET_PASSWORD.getNodeValue())) {
            // 检验用户是否存在
            MemberUserDO userDO = checkUserIfExists(reqVO.getMobile());
            //记录日志
            UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
            userOperationLogCreateReqVO
                    .setUserId(userDO.getId())
                    .setType(UserOperationLogTypeEnum.SEND_CODE.getValue())
                    .setTitle(UserOperationLogTypeEnum.SEND_CODE.getDesc())
                    .setIp(getClientIP())
                    .setContent("修改密码验证码");
            userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);
            //通过memberUserId获取Customer
            CustomerDO customer = customerMapper.getByMemberUserId(userDO.getId());
            if (customer!=null) {
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(1l)
                        .setOperatorName("admin")
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setOperateType(CustomerOperateTypeEnum.VERIFICATION_CODE.getValue())
                        .setRemark("修改密码验证码");
                //记录日志
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        } else if (reqVO.getNodeValue().equals(SmsNodeEnum.MEMBER_LOGIN.getNodeValue())) {
            // 检验用户是否存在
            MemberUserDO userDO = checkUserIfExists(reqVO.getMobile());
            //记录日志
            UserOperationLogCreateReqVO userOperationLogCreateReqVO = new UserOperationLogCreateReqVO();
            userOperationLogCreateReqVO
                    .setUserId(userDO.getId())
                    .setType(UserOperationLogTypeEnum.SEND_CODE.getValue())
                    .setTitle(UserOperationLogTypeEnum.SEND_CODE.getDesc())
                    .setIp(getClientIP())
                    .setContent("登录验证码");
            //通过memberUserId获取Customer
            CustomerDO customer = customerMapper.getByMemberUserId(userDO.getId());
            if (customer != null) {
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(1l)
                        .setOperatorName("admin")
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setOperateType(CustomerOperateTypeEnum.VERIFICATION_CODE.getValue())
                        .setRemark("登录验证码");
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
            //记录日志
            userOperationLogService.createUserOperationLog(userOperationLogCreateReqVO);
        }


        // TODO 要根据不同的场景，校验是否有用户
        smsCodeApi.sendSmsCode(AuthConvert.INSTANCE.convert(reqVO).setCreateIp(getClientIP()));
    }

    @Override
    public void sendEmailCode(Long loginUserId, AppEmailReqVO reqVO) {
        MemberUserDO userVo = userService.getUserByEmail(reqVO.getEmail());
        if (userVo != null) {
            throw exception(USER_EMAIL_ALREADY_EXISTS);
        }
        MemberUserDO user = userService.getUser(loginUserId);
        Map<String, String> tempalteParam = new HashMap<>();
        tempalteParam.put("name", user.getEnglishName());
        //todo 生成验证码逻辑
        tempalteParam.put("code", "9999");
        tempalteParam.put("date", DateUtils.getDate());
        MailTemplateSendReqVO reqVO1 = new MailTemplateSendReqVO();
        reqVO1.setTemplateCode("bindEmail");
        reqVO1.setTemplateParams(tempalteParam);
        reqVO1.setToAddress(reqVO.getEmail());
        mailSendService.sendMail(reqVO1);
    }

    @Override
    public void memberCancellation(Long userId) {
        // 删除用户
        userMapper.deleteById(userId);
        // 清除会员绑定客户信息
        userMapper.unbindingCustomer(userId);
    }

    /**
     * 校验旧密码
     *
     * @param id          用户 id
     * @param oldPassword 旧密码
     * @return MemberUserDO 用户实体
     */
    @VisibleForTesting
    public MemberUserDO checkOldPassword(Long id, String oldPassword) {
        MemberUserDO user = userMapper.selectById(id);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        if (StringUtils.isBlank(user.getPassword())) {
            return user;
        }
        // 参数：未加密密码，编码后的密码
        if (!passwordEncoder.matches(oldPassword, user.getPassword())) {
            throw exception(USER_PASSWORD_FAILED);
        }
        return user;
    }

    /**
     * 校验旧密码
     *
     * @param id          用户 id
     * @param oldPassword 旧密码
     * @return MemberUserDO 用户实体
     */
    @VisibleForTesting
    public MemberUserDO checkOldControlPassword(Long id, String oldPassword) {
        MemberUserDO user = userMapper.selectById(id);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        if (StringUtils.isBlank(user.getControlPassword())) {
            return user;
        }
        // 参数：未加密密码，编码后的密码
        if (!passwordEncoder.matches(oldPassword, user.getControlPassword())) {
            throw exception(USER_PASSWORD_FAILED);
        }
        return user;
    }

    @Override
    public MemberUserDO checkUserIfExists(String mobile) {
        MemberUserDO user = userMapper.selectByMobile(mobile);
        if (user == null) {
            throw exception(USER_NOT_EXISTS);
        }
        return user;
    }

    private void createLogoutLog(Long userId, String username) {
        LoginLogCreateReqDTO reqDTO = new LoginLogCreateReqDTO();
        reqDTO.setLogType(LoginLogTypeEnum.LOGOUT_SELF.getType());
        reqDTO.setTraceId(TracerUtils.getTraceId());
        reqDTO.setUserId(userId);
        reqDTO.setUserType(getUserType().getValue());
        reqDTO.setUsername(username);
        reqDTO.setUserAgent(ServletUtils.getUserAgent());
        reqDTO.setUserIp(getClientIP());
        reqDTO.setResult(LoginResultEnum.SUCCESS.getResult());
        loginLogApi.createLoginLog(reqDTO);
    }


    private void scoreRuleCheck(MemberUserDO user) {
        // 首次登陆发获取注册积分
        if (user.getLoginDate() == null) {
            scoreProducerApi.sendRegisterMessage(ScoreRuleTypeEnum.REGISTER, user.getId(), user.getNickname(), user.getEnglishName(), user.getMobile(), user.getRegisterPlatform());
        }
        // 首次登陆且有推荐码获取推荐积分
        if (user.getLoginDate() == null && user.getReferralCode() != null) {
            scoreProducerApi.sendRecommendMessage(ScoreRuleTypeEnum.RECOMMEND, user.getId(), user.getReferralCode());
        }
    }

}
